package net.citrusleaf.LoadTest;

import net.citrusleaf.CitrusleafClient.*;
import net.citrusleaf.CitrusleafClient;

import java.util.Vector;


public class CLKeyValueStore {
	
	CitrusleafClient cl_client = null;
	String namespace = null;
	String set = null;
	CounterStore counters;
	static int MAX_RECURSION_DEPTH = 10;
	
	public CLKeyValueStore(String hostname, int port, String namespace, String set, CounterStore counters) {
		this.counters = counters;

		cl_client = new CitrusleafClient(hostname, port);
		cl_client.setDefaultNamespace(namespace);
		this.namespace = namespace;
        this.set = set;
        
		if (! cl_client.isConnected()) {
			//System.out.println(String.format("Failed to add %s:%d to cluster.", hostname, port));
			throw new IllegalArgumentException("cannot connect to "+hostname+" on port "+port);
		}		
	}
	
	public ResponseObj SetValue(String key, Vector<ClBin> values, int timeout) {
		return SetValue(key, values, timeout, 0);
	}
	
	protected ResponseObj SetValue(String key, Vector<ClBin> values, int timeout, int recursionDepth) {
		ResponseObj responseObj = new ResponseObj();
		long startTime = System.nanoTime();
		
		if (recursionDepth < MAX_RECURSION_DEPTH) {

			ClOptions clopt = new ClOptions();
			
			if (timeout > 0) {
				clopt.setTimeout(timeout);
			}	
			
			responseObj.rc = cl_client.set(this.namespace, this.set, key, values, clopt, null);
		
			if (responseObj.rc != ClResultCode.OK) {
				// System.out.println("Set failed: rc=" + responseObj.rc + " ns=" + namespace + " set=" + this.set + " key=" + key);
				try {
					Thread.sleep(10);
				} catch (InterruptedException e) {
				}
				responseObj = SetValue(key, values, timeout, recursionDepth + 1);
			}
		}else{
			responseObj.rc = ClResultCode.TIMEOUT; 
		}
		long endTime = System.nanoTime();
		responseObj.td = endTime-startTime;
		return responseObj;
	}

	public ResponseObj IncrementValue(String key, Vector<ClBin> value, int generation, int timeout) {
		return IncrementValue(key, value, generation, timeout, 0);
	}
	
	public ResponseObj IncrementValue(String key, Vector<ClBin> value, int generation, int timeout, int recursionDepth) {
		ResponseObj responseObj = new ResponseObj();
		
		ClOptions clopt = new ClOptions();
		
		long startTime = System.nanoTime();
		
		if (recursionDepth < MAX_RECURSION_DEPTH) {
			if (timeout > 0) {
				clopt.setTimeout(timeout);
			}
		
			ClWriteOptions clw = new ClWriteOptions();
			clw.set_generation(generation);

			responseObj.rc = cl_client.add(this.namespace, this.set, key, value, clopt, clw);
			if (responseObj.rc != ClResultCode.OK) {
				if (responseObj.rc == CitrusleafClient.ClResultCode.GENERATION_ERROR) {
					this.counters.generationErrCnt.incrementAndGet();
				} else {
					try {
						Thread.sleep(10);
					} catch (InterruptedException e) {
					}
					responseObj = IncrementValue(key, value, generation, timeout, recursionDepth + 1);
				}
			}
		} else {
			responseObj.rc = ClResultCode.TIMEOUT; 
		}
		long endTime = System.nanoTime();
		responseObj.td = endTime-startTime;
		return responseObj;
	}

	public ResponseObj GetValue(String key, int timeout) {
		ClResult clr;
		ResponseObj responseObj = new ResponseObj();
		ClOptions clopt = new ClOptions();
		long startTime = System.nanoTime();

		if (timeout > 0)
			clopt.setTimeout(timeout);
		
		clr = cl_client.getAll(this.namespace, this.set, key, clopt);
		if (clr.results != null) {
			Object[] objarr = new Object[clr.results.values().size()];
			if(clr.results.values().size() == 1) {
				objarr[0] = clr.results.values().toArray()[0];
			} else if(clr.results.values().size() > 1) {
				for(int i=0; i<clr.results.size(); i++) {
					objarr[i] = clr.results.get(Integer.toString(i));
				}
			}
			responseObj.value = objarr;
			responseObj.generation = clr.generation;
		} else {
			//System.out.println("GOT NOTHING!");  // XXX check error!!
		}

		long endTime = System.nanoTime();
		responseObj.td = endTime-startTime;
		return responseObj;
	}

	public ResponseObj GetSingleBin(String key, String bin, int timeout) {
		ClResult clr;
		ResponseObj responseObj = new ResponseObj();
		String[] bins = new String[]{bin};
		ClOptions clopt = new ClOptions();
		long startTime = System.nanoTime();

		if (timeout > 0)
			clopt.setTimeout(timeout);
		
		clr = cl_client.get(this.namespace, this.set, key, bins, clopt);
		if (clr.results != null) {
			responseObj.value = clr.results.values().toArray();
			responseObj.generation = clr.generation;
		} // XXX check error

		long endTime = System.nanoTime();
		responseObj.td = endTime-startTime;
		return responseObj;
	}
}
