/*
 *  Citrusleaf Aerospike Java Library
 *
 *  Copyright 2009-2010 by Citrusleaf, Inc.  All rights reserved.
 *  
 *  Availability of this source code to partners and customers includes
 *  redistribution rights covered by individual contract. Please check
 *  your contract for exact rights and responsibilities.
 */
package net.citrusleaf;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.net.UnknownHostException;

import net.citrusleaf.CitrusleafClient.ClResult;
import net.citrusleaf.CitrusleafClient.SerializeException;

public class CLConnection {

	// Need to stash address so I know which queue to put it back on
	InetSocketAddress address;
	
	private Socket socket;
	private InputStream is;
	private OutputStream os;
    public java.util.Date lastUsed; 
    private int timeout;     // this is the default timeout value
    private int set_timeout; // this is what the timeout is currently set to, so as to avoid extra syscalls
    public CLNode node; // just let external users set at will, don't bother with accessors
    
	public CLConnection(InetSocketAddress address) throws UnknownHostException, IOException 
	{
        this.address = address;
        this.socket   = null;
        this.node     = null;
        this.lastUsed = new java.util.Date();
        this.timeout = CitrusleafClient.DEFAULT_TIMEOUT;
        this.set_timeout = -1;
	}
	
	public void connect(int timeout) throws IOException {
		
		socket = new Socket();
		socket.connect(address, timeout);

		socket.setTcpNoDelay(true);
		setTimeout(timeout);

		node.validNetworkActivity();
	}

	public void connect() throws IOException {
		this.connect(this.timeout);
	}
	
	public void close() {
		if (socket == null) return;
		try {
			if (is != null) {
				is.close();
				is = null;
			}
			
			if (os != null) {
				os.close();
				os = null;
			}
			socket.close();			
		} catch (IOException e) {
			if (CitrusleafClient.debugEnabled()) {
				CitrusleafClient.debug("Error closing socket: " + e.getMessage());
			}
		}
		socket = null;
	}
	
	public void poolReturn(boolean connected) {
		
		if (node != null) {
			if (connected == true)
				node.returnConnectionToPool(this);
			else
				node.closeConnection(this);
		}
		else if (socket != null)	{
			if (CitrusleafClient.infoEnabled()) {
				CitrusleafClient.info("pool return but no node, what's up with that");
			}
			try {
				socket.close();
				socket = null;
				
			} catch (IOException e) {
				;
			}
		}
	}
		
	public void setTimeout(int timeout) throws IOException
	{
		try {
			if (this.set_timeout != timeout) {
				socket.setSoTimeout(timeout);
				this.set_timeout = timeout;
			}
		}
		catch (IOException e) {
			if (node==null) 
				node.networkFailure();
			throw e;
		}
	}
		
	public boolean isConnected() 
	{
		if (socket == null) {
			return false;
		}
		return(socket.isConnected());
	}
	

	// so, it's better if the byte buffers are bigger
	// every bytebuffer write actually does a "flush" implicity
	// which bites the big one
	
	public void send(CLBuffer clBuf) throws IOException 
	{
		try {
			os = socket.getOutputStream();
			os.write(clBuf.msg_write_buf, 0, (int)clBuf.msg_write_len);
		}
		catch (IOException e) {
			throw e;
		}
		this.lastUsed = new java.util.Date();
	}


	// This is a little peculiar, stylistically, but only the CL buffer
	// knows the internals of how it's going to read something. In reality,
	// it's going to "hard read" into a small buffer, then do a larger read
	// for the rest if necessary. But that's something only it knows - the connection
	// doesn't! but the connection has the input stream, so there.
	
	public ClResult retrieve(CLBuffer clBuf) throws IOException, SerializeException
	{
		is = socket.getInputStream();
		ClResult rv = null;
		if (clBuf.isReadIterate()) {
			rv = clBuf.multi_get_and_parse(is);
		} else if (clBuf.isBatchRead()) {
			rv = clBuf.get_and_parse_batch(is);
		} else {
			rv = clBuf.get_and_parse(is);
		}
		this.lastUsed = new java.util.Date();
		return rv;
	}
	
	public void reportNetworkFailure() {
		if (node!=null) node.networkFailure();	
	}
	
	public void reportNetworkSuccess() {
		if (node != null) node.validNetworkActivity();
	}
}
