package net.citrusleaf.examples;

import java.util.ArrayList;
import java.util.List;

import net.citrusleaf.CitrusleafClient;
import net.citrusleaf.CitrusleafClient.ClBin;
import net.citrusleaf.CitrusleafClient.ClResult;
import net.citrusleaf.CitrusleafClient.ClResultCode;

public class Add extends Example {

	public Add(Console console) {
		super(console);
	}

	/**
	 * Add integer values.
	 */
	@Override
	public void runExample(CitrusleafClient cs, Parameters params) throws Exception {
		String key = "addkey";
		String bin = params.getBinName("addbin");  
		int value = 10;

		// Delete record if it already exists.
		ClResultCode r = cs.delete(params.namespace, params.set, key, null, null);

		if (! (r == ClResultCode.OK || r == ClResultCode.KEY_NOT_FOUND_ERROR)) {
			throw new Exception(String.format("Failed to delete: ns=%s set=%s key=%s code=%s",
				params.namespace, params.set, key, r));
		}

		// Perform some adds and check results.
		console.info("Initial add will create record.  Initial value is " + value + '.');
		r = cs.add(params.namespace, params.set, key, bin, value, null, null);

		if (r != ClResultCode.OK) {
			throw new Exception(String.format(
				"Failed to add: ns=%s set=%s key=%s bin=%s value=%s code=%s", 
				params.namespace, params.set, key, bin, value, r));
		}

		value = 5;
		console.info("Add " + value + " to existing record.");
		r = cs.add(params.namespace, params.set, key, bin, value, null, null);

		if (r != ClResultCode.OK) {
			throw new Exception(String.format(
				"Failed to add: ns=%s set=%s key=%s bin=%s value=%s code=%s", 
				params.namespace, params.set, key, bin, value, r));
		}

		int expected = 15;
		ClResult result = cs.get(params.namespace, params.set, key, bin, null);

		if (result.resultCode != ClResultCode.OK) {
			throw new Exception(String.format(
				"Failed to get: namespace=%s set=%s key=%s bin=%s code=%s", 
				params.namespace, params.set, key, bin, result.resultCode));
		}

		// The value received from the server is an unsigned byte stream.
		// Convert to an integer before comparing with expected.
		int received = (Integer)result.result;

		if (received == expected) {
			console.info("Add successful: ns=%s set=%s key=%s bin=%s value=%s", 
				params.namespace, params.set, key, bin, received);
		}
		else {
			console.error("Add mismatch: Expected %d. Received %d.", expected, received);
		}

		// Demonstrate add and get combined command.
		value = 30;
		console.info("Add " + value + " to existing record.");
		List<ClBin> bins = new ArrayList<ClBin>();
		bins.add(new ClBin(bin, value));
		result = cs.addAndGet(params.namespace, params.set, key, bins, null, null);

		if (result.resultCode != ClResultCode.OK) {
			throw new Exception(String.format(
				"Failed to get: namespace=%s set=%s key=%s bin=%s code=%s", 
				params.namespace, params.set, key, bin, result.resultCode));
		}

		expected = 45;
		received = (Integer)result.results.get(bin);

		if (received == expected) {
			console.info("Add successful: ns=%s set=%s key=%s bin=%s value=%s", 
				params.namespace, params.set, key, bin, received);
		}
		else {
			console.error("Add mismatch: Expected %d. Received %d.", expected, received);
		}
	}
}
