package net.citrusleaf.examples;

import java.util.ArrayList;
import java.util.List;

import net.citrusleaf.CitrusleafClient;
import net.citrusleaf.CitrusleafClient.ClResult;
import net.citrusleaf.CitrusleafClient.ClResultCode;

public class BatchGet extends Example {

	public BatchGet(Console console) {
		super(console);
	}

	/**
	 * Batch multiple gets in one call to the server.
	 */
	@Override
	public void runExample(CitrusleafClient cs, Parameters params) throws Exception {
		int size = 8;
		String keyPrefix = "batchkey";
		String bin = params.getBinName("batchbin");  
		String valuePrefix = "batchvalue";
		writeRecords(cs, params.namespace, params.set, keyPrefix, bin, valuePrefix, size);
		batchReads(cs, params.namespace, params.set, keyPrefix, bin, size);
	}

	/**
	 * Write records individually.
	 */
	private void writeRecords(
		CitrusleafClient cs, 
		String ns, 
		String set,
		String keyPrefix,
		String bin,
		String valuePrefix,
		int size
	) throws Exception {
		for (int i = 1; i <= size; i++) {
			String key = keyPrefix + i;
			String value = valuePrefix + i;
			ClResultCode r = cs.set(ns, set, key, bin, value, null, null);

			if (r != ClResultCode.OK) {
				throw new Exception(String.format(
					"Failed to set: ns=%s set=%s key=%s bin=%s value=%s code=%s",
					ns, set, key, bin, value, r));
			}

			console.info("Set successful: ns=%s set=%s key=%s bin=%s value=%s",
				ns, set, key, bin, value);
		}
	}

	/**
	 * Read records in one batch.
	 */
	private void batchReads (
		CitrusleafClient cs, 
		String ns, 
		String set,
		String keyPrefix,
		String bin,
		int size
	) throws Exception {
		// Batch gets into one call.
		List<Object> keys = new ArrayList<Object>();
		for (int i = 1; i <= size; i++) {
			keys.add(keyPrefix + i);
		}

		ClResult[] results = cs.batchGet(ns, set, keys, bin, null);
		int count = 0;

		for (ClResult result : results) {
			Object key = keys.get(count++);

            if (result.resultCode == ClResultCode.OK) {
                Object value = result.results.get(bin);
                console.info("Get successful: ns=%s set=%s key=%s bin=%s value=%s",
                        ns, set, key, bin, value);                        	
            }
            else {
            	console.error("Failed to get: ns=%s set=%s key=%s bin=%s code=%s",
                        ns, set, key, bin, result.resultCode);
            }
        }
    }
}
