package net.citrusleaf.examples;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import net.citrusleaf.CitrusleafClient;
import net.citrusleaf.CitrusleafClient.ClBin;
import net.citrusleaf.CitrusleafClient.ClResult;
import net.citrusleaf.CitrusleafClient.ClResultCode;

public class DeleteBin extends Example {

	public DeleteBin(Console console) {
		super(console);
	}

	/**
	 * Drop a bin from a record.
	 */
	@Override
	public void runExample(CitrusleafClient cs, Parameters params) throws Exception {
		if (params.singleBin) {
			console.info("Delete bin is not application to single bin servers.");			
			return;
		}
			
		console.info("Write multi-bin record.");

		String key = "delbinkey";
		List<ClBin> bins = new ArrayList<ClBin>(2);
		bins.add(new ClBin("bin1", "value1"));
		bins.add(new ClBin("bin2", "value2"));
	
		ClResultCode r = cs.set(params.namespace, params.set, key, bins, null, null);

		if (r != ClResultCode.OK) {
			throw new Exception(String.format(
				"Failed to set: namespace=%s set=%s key=%s code=%s", 
				params.namespace, params.set, key, r));
		}

		console.info("Delete one bin in the record.");
		String bin = "bin1";
		String value = null;  // Set bin value to null to drop bin.
		
		r = cs.set(params.namespace, params.set, key, bin, value, null, null);

		if (r != ClResultCode.OK) {
			throw new Exception(String.format(
				"Failed to set: namespace=%s set=%s key=%s bin=%s value=%s code=%s", 
				params.namespace, params.set, key, bin, value, r));
		}

		console.info("Read record.");
		List<String> binNames = new ArrayList<String>(bins.size());
		for (ClBin item : bins) {
			binNames.add(item.name);
		}
		binNames.add("bin3");
		
		ClResult result = cs.get(params.namespace, params.set, key, binNames, null);

		if (result.resultCode != ClResultCode.OK) {
			throw new Exception(String.format(
				"Failed to get: namespace=%s set=%s key=%s code=%s", 
				params.namespace, params.set, key, result.resultCode));
		}

		Map<String,Object> map = result.results;
		for (Map.Entry<String,Object> entry : result.results.entrySet()) {
			console.info("Received: namespace=%s set=%s key=%s bin=%s value=%s", 
					params.namespace, params.set, key, entry.getKey(), entry.getValue());			
		}
		
		boolean valid = true;
		
		if (map.get("bin1") != null) {
			console.error("bin1 still exists.");
			valid = false;
		}
		
		Object v2 = map.get("bin2");
		
		if (v2 == null || ! v2.equals("value2")) {
			console.error("bin2 value mismatch.");
			valid = false;
		}
		
		if (valid) {
			console.info("Bin delete successful");
		}
	}
}
