package net.citrusleaf.examples;

import net.citrusleaf.CitrusleafClient;
import net.citrusleaf.CitrusleafClient.ClResult;
import net.citrusleaf.CitrusleafClient.ClResultCode;
import net.citrusleaf.CitrusleafClient.ClWriteOptions;

public class Expire extends Example {

	public Expire(Console console) {
		super(console);
	}

	/**
	 * Write and twice read a bin value, demonstrating record expiration.
	 */
	@Override
	public void runExample(CitrusleafClient cs, Parameters params) throws Exception {
		String key = "expirekey";
		String bin = params.getBinName("expirebin");  
		String value = "expirevalue";

		// Specify that record expires 2 seconds after it's written.
		ClWriteOptions wOpts = new ClWriteOptions();
		wOpts.expiration = 2;

		ClResultCode r = cs.set(params.namespace, params.set, key, bin, value, null, wOpts);

		if (r != ClResultCode.OK) {
			throw new Exception(String.format(
				"Failed to set: namespace=%s set=%s key=%s bin=%s value=%s code=%s",
				params.namespace, params.set, key, bin, value, r));
		}

		console.info("Set successful: namespace=%s set=%s key=%s bin=%s value=%s",
			params.namespace, params.set, key, bin, value);

		// Read the record before it expires, showing it's there.
		ClResult result = cs.get(params.namespace, params.set, key, bin, null);

		if (result.resultCode != ClResultCode.OK) {
			throw new Exception(String.format(
				"Failed to get: namespace=%s set=%s key=%s bin=%s code=%s",
				params.namespace, params.set, key, bin, result.resultCode));
		}

		if (value.equals(result.result)) {
			console.info("Get successful: namespace=%s set=%s key=%s bin=%s value=%s",
				params.namespace, params.set, key, bin, result.result);
		}
		else {
			throw new Exception(String.format(
				"Set/Get mismatch: Expected %s. Received %s.",
				value, result.result));
		}

		// Read the record after it expires, showing it's gone.
		console.info("Sleeping for 3 seconds ...");
		Thread.sleep(3 * 1000);
		result = cs.get(params.namespace, params.set, key, bin, null);

		if (result.resultCode != ClResultCode.KEY_NOT_FOUND_ERROR) {
			throw new Exception(String.format(
				"Did not expect to get: namespace=%s set=%s key=%s bin=%s code=%s",
				params.namespace, params.set, key, bin, result.resultCode));
		}

		console.info("Expiry successful: namespace=%s set=%s key=%s bin=%s code=%s",
			params.namespace, params.set, key, bin, result.resultCode);
	}
}
