package net.citrusleaf.examples;

import net.citrusleaf.CitrusleafClient;
import net.citrusleaf.CitrusleafClient.ClResult;
import net.citrusleaf.CitrusleafClient.ClResultCode;
import net.citrusleaf.CitrusleafClient.ClWriteOptions;

public class Generation extends Example {

	public Generation(Console console) {
		super(console);
	}

	/**
	 * Exercise record generation functionality.
	 */
	@Override
	public void runExample(CitrusleafClient cs, Parameters params) throws Exception {
		String key = "genkey";
		String bin = params.getBinName("genbin");  
		String value = "genvalue1";

		// Delete record if it already exists.
		ClResultCode r = cs.delete(params.namespace, params.set, key, null, null);

		if (! (r == ClResultCode.OK || r == ClResultCode.KEY_NOT_FOUND_ERROR)) {
			throw new Exception(String.format("Failed to delete: ns=%s set=%s key=%s code=%s",
				params.namespace, params.set, key, r));
		}

		// Set some values for the same record.
		console.info("Set: namespace=%s set=%s key=%s bin=%s value=%s",
				params.namespace, params.set, key, bin, value);

		r = cs.set(params.namespace, params.set, key, bin, value, null, null);

		if (r != ClResultCode.OK) {
			throw new Exception(String.format(
				"Failed to set: namespace=%s set=%s key=%s bin=%s value=%s code=%s", 
				params.namespace, params.set, key, bin, value, r));
		}

		value = "genvalue2";
		console.info("Set: namespace=%s set=%s key=%s bin=%s value=%s",
				params.namespace, params.set, key, bin, value);

		r = cs.set(params.namespace, params.set, key, bin, value, null, null);

		if (r != ClResultCode.OK) {
			throw new Exception(String.format(
				"Failed to set: namespace=%s set=%s key=%s bin=%s value=%s code=%s", 
				params.namespace, params.set, key, bin, value, r));
		}

		// Retrieve record and its generation count.
		ClResult result = cs.get(params.namespace, params.set, key, bin, null);

		if (result.resultCode != ClResultCode.OK) {
			throw new Exception(String.format(
				"Failed to get: namespace=%s set=%s key=%s bin=%s code=%s", 
				params.namespace, params.set, key, bin, result.resultCode));
		}

		if (result.result.equals(value)) {
			console.info("Get successful: ns=%s set=%s key=%s bin=%s value=%s generation=%d", 
				params.namespace, params.set, key, bin, result.result, result.generation);
		}
		else {
			throw new Exception(String.format(
				"Get mismatch: Expected %s. Received %s.", value, result.result));
		}

		// Set record and fail if it's not the expected generation.
		value = "genvalue3";
		int generation = result.generation;
		console.info("Set: namespace=%s set=%s key=%s bin=%s value=%s expected generation=%d",
			params.namespace, params.set, key, bin, value, generation);

		ClWriteOptions writeInfo = new ClWriteOptions();
		writeInfo.set_generation(generation);
		r = cs.set(params.namespace, params.set, key, bin, value, null, writeInfo);

		if (r != ClResultCode.OK) {
			throw new Exception(String.format(
				"Failed to set: namespace=%s set=%s key=%s bin=%s value=%s code=%s", 
				params.namespace, params.set, key, bin, value, r));
		}

		// Set record with invalid generation and check results .
		value = "genvalue4";
		generation = 9999;
		console.info("Set: namespace=%s set=%s key=%s bin=%s value=%s expected generation=%d",
			params.namespace, params.set, key, bin, value, generation);

		writeInfo.set_generation(generation);
		r = cs.set(params.namespace, params.set, key, bin, value, null, writeInfo);

		if (r == ClResultCode.GENERATION_ERROR) {
			console.info("Success: Generation error returned as expected.");
		}
		else {
			throw new Exception(String.format(
				"Unexpected set return code: namespace=%s set=%s key=%s bin=%s value=%s code=%s", 
				params.namespace, params.set, key, bin, value, r));
		}

		// Verify results.
		result = cs.get(params.namespace, params.set, key, bin, null);

		if (result.resultCode != ClResultCode.OK) {
			throw new Exception(String.format(
				"Failed to get: namespace=%s set=%s key=%s bin=%s code=%s", 
				params.namespace, params.set, key, bin, result.resultCode));
		}

		value = "genvalue3";
		if (result.result.equals(value)) {
			console.info("Get successful: ns=%s set=%s key=%s bin=%s value=%s generation=%d", 
				params.namespace, params.set, key, bin, result.result, result.generation);
		}
		else {
			throw new Exception(String.format(
				"Get mismatch: Expected %s. Received %s.", value, result.result));
		}
	}
}
