package net.citrusleaf.examples;

import net.citrusleaf.CitrusleafClient;
import net.citrusleaf.CitrusleafClient.ClResult;
import net.citrusleaf.CitrusleafClient.ClResultCode;

public class Prepend extends Example {

	public Prepend(Console console) {
		super(console);
	}

	/**
	 * Prepend string to an existing string.
	 */
	@Override
	public void runExample(CitrusleafClient cs, Parameters params) throws Exception {
		String key = "prependkey";
		String bin = params.getBinName("prependbin");  
		String value = "World";

		// Delete record if it already exists.
		ClResultCode r = cs.delete(params.namespace, params.set, key, null, null);

		if (! (r == ClResultCode.OK || r == ClResultCode.KEY_NOT_FOUND_ERROR)) {
			throw new Exception(String.format("Failed to delete: ns=%s set=%s key=%s code=%s",
				params.namespace, params.set, key, r));
		}

		console.info("Initial prepend will create record.  Initial value is " + value + '.');
		r = cs.prepend(params.namespace, params.set, key, bin, value, null, null);

		if (r != ClResultCode.OK) {
			throw new Exception(String.format(
				"Failed to prepend: ns=%s set=%s key=%s bin=%s value=%s code=%s", 
				params.namespace, params.set, key, bin, value, r));
		}

		value = "Hello ";
		console.info("Prepend \"" + value + "\" to existing record.");
		r = cs.prepend(params.namespace, params.set, key, bin, value, null, null);

		if (r != ClResultCode.OK) {
			throw new Exception(String.format(
				"Failed to prepend: ns=%s set=%s key=%s bin=%s value=%s code=%s", 
				params.namespace, params.set, key, bin, value, r));
		}

		ClResult result = cs.get(params.namespace, params.set, key, bin, null);

		if (result.resultCode != ClResultCode.OK) {
			throw new Exception(String.format(
				"Failed to get: namespace=%s set=%s key=%s bin=%s code=%s", 
				params.namespace, params.set, key, bin, result.resultCode));
		}

		// The value received from the server is an unsigned byte stream.
		// Convert to an integer before comparing with expected.
		Object received = result.result;
		String expected = "Hello World";

		if (received.equals(expected)) {
			console.info("Prepend successful: ns=%s set=%s key=%s bin=%s value=%s", 
				params.namespace, params.set, key, bin, received);
		}
		else {
			console.error("Prepend mismatch: Expected %s. Received %s.", expected, received);
		}
	}
}
