package net.citrusleaf.examples;

import java.util.Map;

import net.citrusleaf.CitrusleafClient;
import net.citrusleaf.CitrusleafClient.ClResultCode;
import net.citrusleaf.CitrusleafClient.ScanCallback;

public class ScanParallel extends Example implements ScanCallback {

	private int recordCount = 0;

	public ScanParallel(Console console) {
		super(console);
	}

	/**
	 * Scan all nodes in parallel and read all records in a set.
	 */
	@Override
	public void runExample(CitrusleafClient cs, Parameters params) throws Exception {
		console.info("Scan database.");
		recordCount = 0;
		long begin = System.currentTimeMillis();
		ClResultCode r = cs.scan(params.namespace, params.set, this, null);

		if (r != ClResultCode.OK) {
			throw new Exception(String.format(
				"Failed to scan: ns=%s set=%s code=%s",
				params.namespace, params.set, r));
		}
		long end = System.currentTimeMillis();
		double seconds =  (double)(end - begin) / 1000.0;
		console.info("Total records returned: " + recordCount);
		console.info("Elapsed time: " + seconds + " seconds");
		double performance = Math.round((double)recordCount / seconds);
		console.info("Records/second: " + performance);
	}

	@Override
	public void scanCallback(
		String namespace, 
		String set, 
		byte[] digest,
		Map<String, Object> bins,
		int generation,
		int expirationDate,
		Object userData
	) {
		recordCount++;

		if ((recordCount % 10000) == 0) {
			console.info("Records " + recordCount);
		}
	}		
}
