package net.citrusleaf.examples;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.citrusleaf.CitrusleafClient;
import net.citrusleaf.CitrusleafClient.ClOptions;
import net.citrusleaf.CitrusleafClient.ClResultCode;
import net.citrusleaf.CitrusleafClient.ClScanningOptions;
import net.citrusleaf.CitrusleafClient.ClScanningPriority;
import net.citrusleaf.CitrusleafClient.ScanCallback;

public class ScanSeries extends Example implements ScanCallback {

	private Map<String,Metrics> setMap = new HashMap<String,Metrics>();

	public ScanSeries(Console console) {
		super(console);
	}

	/**
	 * Scan all nodes in series and read all records in all sets.
	 */
	@Override
	public void runExample(CitrusleafClient cs, Parameters params) throws Exception {
		console.info("Scan namespace " + params.namespace);
		
		// Use low scan priority.  This will take more time, but it will reduce
		// the load on the server.
		ClScanningOptions scanOpts = new ClScanningOptions();
		scanOpts.setPriority(ClScanningPriority.LOW);

		// Do not timeout on scan.
		ClOptions opts = new ClOptions();
		opts.setOneShot();
		opts.setTimeout(0);
		
		List<String> nodeList = cs.getNodeNameList();
		long begin = System.currentTimeMillis();
		
		for (String nodeName : nodeList) {
			console.info("Scan node " + nodeName);
			ClResultCode r = cs.scanNode(nodeName, params.namespace, null, true, 100, opts, scanOpts, this, null);

			if (r != ClResultCode.OK) {
				throw new Exception(String.format(
					"Failed to scan: node=%s ns=%s code=%s",
					nodeName, params.namespace, r));
			}
			
			for (Map.Entry<String,Metrics> entry : setMap.entrySet()) {
				console.info("Node " + nodeName + " set " + entry.getKey() + " count: " +  entry.getValue().count);
				entry.getValue().count = 0;
			}
		}

		long end = System.currentTimeMillis();
		double seconds =  (double)(end - begin) / 1000.0;
		console.info("Elapsed time: " + seconds + " seconds");
		
		long total = 0;
		
		for (Map.Entry<String,Metrics> entry : setMap.entrySet()) {
			console.info("Total set " + entry.getKey() + " count: " +  entry.getValue().total);
			total += entry.getValue().total;
		}
		console.info("Grand total: " + total);
		double performance = Math.round((double)total / seconds);
		console.info("Records/second: " + performance);
	}

	@Override
	public void scanCallback(
		String namespace, 
		String set, 
		byte[] digest,
		Map<String, Object> bins,
		int generation,
		int expirationDate,
		Object userData
	) {
		Metrics metrics = setMap.get(set);
		
		if (metrics == null) {
			metrics = new Metrics();
		}
		metrics.count++;
		metrics.total++;
		setMap.put(set, metrics);
	}
	
	private static class Metrics {
		public long count = 0;
		public long total = 0;
	}
}
