package net.citrusleaf.examples;

import net.citrusleaf.CitrusleafClient;
import net.citrusleaf.CitrusleafClient.ClResult;
import net.citrusleaf.CitrusleafClient.ClResultCode;

public class Serialize extends Example {

	public Serialize(Console console) {
		super(console);
	}

	/**
	 * Write complex object using serializer.
	 */
	@Override
	public void runExample(CitrusleafClient cs, Parameters params) throws Exception {
		String key = "serializetestserializetestserializetest";
		String bin = params.getBinName("serialbin");  

		// Delete record if it already exists.
		ClResultCode r = cs.delete(params.namespace, params.set, key, null, null);

		if (! (r == ClResultCode.OK || r == ClResultCode.KEY_NOT_FOUND_ERROR)) {
			throw new Exception(String.format("Failed to delete: ns=%s set=%s key=%s code=%s",
				params.namespace, params.set, key, r));
		}

		console.info("Initialize array");

		int[] array = new int[10000];

		for (int i = 0; i < 10000; i++) {
			array[i] = i * i;
		}

		// Do a test that pushes this complex object through the serializer
		console.info("Write array using serializer.");
		r  = cs.set(params.namespace, params.set, key, bin, array, null, null);

		if (r != ClResultCode.OK) {
			throw new Exception(String.format(
				"Failed to set: namespace=%s set=%s key=%s bin=%s code=%s", 
				params.namespace, params.set, key, bin, r));
		}

		console.info("Read array using serializer.");
		ClResult result = cs.get(params.namespace, params.set, key, bin, null);

		if (result.resultCode != ClResultCode.OK) {
			throw new Exception(String.format(
				"Failed to get: namespace=%s set=%s key=%s bin=%s code=%s", 
				params.namespace, params.set, key, bin, result.resultCode));
		}

		int[] received;

		try {
			received = (int[])result.results.get(bin);
		}
		catch (Exception e) {
			throw new Exception(String.format(
				"Failed to parse returned value: namespace=%s set=%s key=%s bin=%s code=%s", 
				params.namespace, params.set, key, bin, result.resultCode));
		}

		if (received.length != 10000) {
			throw new Exception(String.format(
				"Array length mismatch: Expected=%d Received=%d", 10000, received.length));
		}

		for (int i = 0; i < 10000; i++) {
			if (received[i] != i * i) {
				throw new Exception(String.format(
					"Mismatch: index=%d expected=%d received=%d", i, received[i], i*i)); 
			}
		}

		console.info("Read successful.");
	}
}
