package net.citrusleaf.examples;

import net.citrusleaf.CitrusleafClient;
import net.citrusleaf.CitrusleafClient.ClResult;
import net.citrusleaf.CitrusleafClient.ClResultCode;

public class SetGet extends Example {

	public SetGet(Console console) {
		super(console);
	}

	/**
	 * Write and read a bin value.
	 */
	@Override
	public void runExample(CitrusleafClient cs, Parameters params) throws Exception {
		String key = "setgetkey";
		String bin = params.getBinName("setgetbin");  
		String value = "setgetvalue";

		ClResultCode r = cs.set(params.namespace, params.set, key, bin, value, null, null);

		if (r != ClResultCode.OK) {
			throw new Exception(String.format(
				"Failed to set: namespace=%s set=%s key=%s bin=%s value=%s code=%s", 
				params.namespace, params.set, key, bin, value, r));
		}

		console.info("Set successful: namespace=%s set=%s key=%s bin=%s value=%s",
			params.namespace, params.set, key, bin, value);

		ClResult result = cs.get(params.namespace, params.set, key, bin, null);

		if (result.resultCode != ClResultCode.OK) {
			throw new Exception(String.format(
				"Failed to get: namespace=%s set=%s key=%s bin=%s code=%s", 
				params.namespace, params.set, key, bin, result.resultCode));
		}

		if (value.equals(result.result)) {
			console.info("Get successful: namespace=%s set=%s key=%s bin=%s value=%s", 
				params.namespace, params.set, key, bin, result.result);
		}
		else {
			console.error("Set/Get mismatch: Expected %s. Received %s.", 
				value, result.result);
		}
	}
}
