package net.citrusleaf.examples;

import java.util.ArrayList;
import java.util.List;

import net.citrusleaf.CitrusleafClient;
import net.citrusleaf.CitrusleafClient.ClResult;
import net.citrusleaf.CitrusleafClient.ClResultCode;
import net.citrusleaf.CitrusleafClient.ClWriteOptions;

public class Touch extends Example {

	public Touch(Console console) {
		super(console);
	}

	/**
	 * Add integer values.
	 */
	@Override
	public void runExample(CitrusleafClient cs, Parameters params) throws Exception {
		String key = "touchkey";
		String bin = params.getBinName("touchbin");  
		String value = "touchvalue";

		console.info("Create record with 2 second expiration.");
		ClWriteOptions writeInfo = new ClWriteOptions();
		writeInfo.expiration = 2;
		ClResultCode r = cs.set(params.namespace, params.set, key, bin, value, null, writeInfo);

		if (r != ClResultCode.OK) {
			throw new Exception(String.format(
				"Failed to set: namespace=%s set=%s key=%s bin=%s value=%s code=%s", 
				params.namespace, params.set, key, bin, value, r));
		}

		console.info("Touch same record with 5 second expiration.");
		List<String> binNames = new ArrayList<String>();
		binNames.add(bin);

		ClResult result = cs.getWithTouch(params.namespace, params.set, key, binNames, 5, null);

		if (result.resultCode != ClResultCode.OK) {
			throw new Exception(String.format(
				"Failed getWithTouch: namespace=%s set=%s key=%s bin=%s code=%s", 
				params.namespace, params.set, key, bin, result.resultCode));
		}

		console.info("Sleep 3 seconds.");
		Thread.sleep(3000);

		result = cs.get(params.namespace, params.set, key, bin, null);

		if (result.resultCode != ClResultCode.OK) {
			throw new Exception(String.format(
				"Failed to get: namespace=%s set=%s key=%s bin=%s code=%s", 
				params.namespace, params.set, key, bin, result.resultCode));
		}

		console.info("Success. Record still exists.");
		console.info("Sleep 4 seconds.");
		Thread.sleep(4000);

		result = cs.get(params.namespace, params.set, key, bin, null);

		switch (result.resultCode) {
			case OK:
				console.error("Found record when it should have expired.");
				break;

			case KEY_NOT_FOUND_ERROR:
				console.info("Success. Record expired as expected.");
				break;

			default:
				throw new Exception(String.format(
					"Failed to get: ns={0} set={1} key={2} bin={3} code={4}", 
					params.namespace, params.set, key, bin, r));
		}
	}
}
