package com.citrusleaf;

import java.io.IOException;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import net.citrusleaf.CitrusleafClient;
import net.citrusleaf.CitrusleafClient.ClBin;
import net.citrusleaf.CitrusleafClient.ClOptions;
import net.citrusleaf.CitrusleafClient.ClResult;
import net.citrusleaf.CitrusleafClient.ClResultCode;
import net.citrusleaf.CitrusleafClient.ClWriteOptions;

/**
 * Copyright 2012 Citrusleaf
 * Author: CSW
 * Date: Jun 20, 2012
 * Time: 4:15:30 PM
 */
public class CitrusleafServlet extends HttpServlet 
{
	private static final long serialVersionUID = 1L;

	protected CitrusleafClient cc;
    protected String host;
    protected int port;

    @Override
	protected void service(HttpServletRequest request, HttpServletResponse response) 
		throws ServletException, IOException {
    	
    	try {
            String method = request.getMethod();

            if (method.equals("OPTIONS")) {
                // Available HTTP options for this command
                response.setHeader("Access-Control-Allow-Origin", "*");
                response.setHeader("Access-Control-Allow-Methods", "DELETE,OPTIONS,GET,POST");
                response.setHeader("Access-Control-Max-Age", "1000");
                response.setStatus(HttpServletResponse.SC_OK);
                return;
            }

            if (! initConnection(request, response))
				return;

            processRequest(request, response);
    	}
    	catch (Exception e) {
    		throw new ServletException(e);
    	}
    }
    
	private boolean initConnection(HttpServletRequest request, HttpServletResponse response) 
		throws IOException {

		String hostString = request.getParameter("host");
		String portString = request.getParameter("port");
				
		if (cc != null && cc.isConnected() && hostString != null && portString != null && host.equals(hostString) && Integer.toString(port).equals(portString))
			return true;
				
		host = hostString;
		
	    if (host == null || host.length() == 0) {
	    	try {
		        host = getServletConfig().getInitParameter("clusterAddr").trim();
		    } 
		    catch (Exception e) {
		        host = "127.0.0.1";
		    }
	    }
	    
		if (portString == null || portString.length() == 0) {
		    try {
		        portString = getServletConfig().getInitParameter("clusterPort");
		    } 
		    catch (Exception e) {
		    	portString = "3000";
		    }
	    }
		
		try {
			port = Integer.parseInt(portString);
		}
		catch (Exception e) {
			displayLoginError(response, host, portString);	
			return false;
		}
		
		cc = new CitrusleafClient(host, port);						
		
		if (cc.isConnected())
			return true;
		
		displayLoginError(response, host, portString);	
		return false;
	}
	
	private void displayLoginError(HttpServletResponse response, String hostString, String portString) 
		throws IOException {
        
		String errorMsg = "Invalid Citrusleaf server: " + hostString + ':' + portString;
		response.sendError(HttpServletResponse.SC_NOT_FOUND, errorMsg); 
	}
	
	private void processRequest(HttpServletRequest request, HttpServletResponse response) 
		throws ServletException, IOException {

        String method = request.getMethod();
        String jsonpCallback = null;

        response.setHeader("Access-Control-Allow-Origin", "*");
        response.setContentType("application/json");
        response.setHeader("Cache-Control", "no-cache");

        @SuppressWarnings("unchecked")
		Map<String, String[]> params = request.getParameterMap();
        String pathInfo = request.getPathInfo();

        final Writer writer = response.getWriter();

        // If no pathInfo, just bail now.
        if (pathInfo == null) {
            response.setStatus(HttpServletResponse.SC_OK);
            writer.flush();
            return;
        }

        // Method override - so handy when debugging, or using a brain-dead client
        if (params.containsKey("X-method")) {
            method = params.get("X-method")[0];
        }

        int responseCode = 200;
        String errorMsg = "";
        try {
            pathInfo = stripChar(pathInfo, '/');
            String[] pathArray = pathInfo.split("/");

            // Get namespace, set, and key from the path.
            // Path *must* be of form namespace/set/key
            if (pathArray.length != 3) {
                throw new ResourceNotFoundException("Must Specify Namespace, Key, and Value");
            }

            String namespace = pathArray[0];
            String set       = pathArray[1];
            String key       = pathArray[2];

            // Get other parameters from the parameters on the URL
            String[] values      = params.get("value");
            String[] bins        = params.get("bin");
            String[] generations = params.get("generation");
            String[] timeouts    = params.get("timeout");
            String[] ops         = params.get("op");
            String[] expirations = params.get("expiration");
            String[] callbacks   = params.get("callback");
            String[] jsonps      = params.get("jsonp");

            int timeout = 0;
            try {
                timeout = (timeouts == null || timeouts.length == 0) ? 0 : Integer.parseInt(timeouts[0]);
            }catch (NumberFormatException e) {
                log("bad timeout received");
                throw new BadParamException("Timeout Not Numeric");
            }

            int expiration = 0;
            try {
                expiration = (expirations == null || expirations.length == 0) ? 0 : Integer.parseInt(expirations[0]);
            }catch (NumberFormatException e) {
                log("bad expiration received");
                throw new BadParamException("Expiration Not Numeric");
            }

            int generation = 0;
            try {
                generation = (generations == null || generations.length == 0) ? 0 : Integer.parseInt(generations[0]);
            }catch (NumberFormatException e) {
                log("bad generation received");
                throw new BadParamException("Generation Not Numeric");
            }

            if (callbacks != null) {
                jsonpCallback = callbacks[0];
            } else if (jsonps != null) {
                jsonpCallback = jsonps[0];
            }

            ClOptions cl_opts = new ClOptions();
            if (timeout != 0) {
                cl_opts.setTimeout(timeout);
            }

            if (method.equals("GET")) {
                ClResult result;
                // check that we've got a key
                if (key == null) {
                    log("bad key received");
                    throw new BadParamException("No Key");
                }
                if (bins == null)
                    result = this.cc.getAll(namespace, set, key, cl_opts);
                else
                    result = this.cc.get(namespace, set, key, bins, cl_opts);
                
                if (result.resultCode == ClResultCode.OK) {
                    if (result.result != null) {
                        //String resultJSON = "{\"result\" :" + '"' + result.result.toString() + '"' + "}";
                    } else if (result.results != null) {
                        Set<?> keyset = result.results.keySet();
                        Iterator<?> iterator = keyset.iterator();
                        if (!iterator.hasNext()) {
                            throw new ResourceNotFoundException("Server Returns No Values");
                        }

                        String resultJSON = "{\"result\" : {";
                        while (iterator.hasNext()) {
                            Object binName = iterator.next();
                            Object binValue = result.results.get(binName);
                            resultJSON += "\"" + binName.toString() + "\" : ";
                            if( binValue instanceof Integer ) {
                                resultJSON += binValue.toString();
                            } else if (binValue instanceof byte[]) {
                                System.out.println(" it's a blob!");
                            } else if (binValue instanceof String) {
                                resultJSON += "\"" +  binValue.toString() + "\"";
                            } else {
                                resultJSON += "\"\"";
                            }
                            if (iterator.hasNext()) {
                                resultJSON += ",";
                            }
                        }
                        resultJSON += "},";
                        resultJSON += "\"generation\" : " + result.generation + "}";
                        if (jsonpCallback != null) {
                            resultJSON = jsonpCallback + "(" + resultJSON + ")";
                            response.setContentType("application/javascript");
                        }
                        writer.write(resultJSON);
                    }
                } else {
                	throwOnError(result.resultCode); 
                }

            }else if (method.equals("POST")) {
            
                if (values == null || values.length < 1) {
                    throw new BadParamException("Setting a key requires a value");
                }

                boolean isArithmeticAdd = false;
                boolean isAppend        = false;
                boolean isPrepend       = false;

                if (ops != null && ops.length > 0) {
                 	if (ops[0].equals("add")) {
                    	isArithmeticAdd = true;
                    } else if (ops[0].equals("append")){
                    	isAppend  = true;
                    } else if (ops[0].equals("prepend")){
                    	isPrepend = true;
                    }
                }
                
                ClWriteOptions cl_wr_opts = new ClWriteOptions();
                cl_wr_opts.expiration = expiration;
                if (generation != 0) {
                    cl_wr_opts.set_generation(generation);
                }

                ClResultCode rc;
                if (values.length > 1) {
                    if (bins == null || bins.length != values.length) {
                        throw new BadParamException("Setting a key requires a bin name");
                    }
                    ArrayList <ClBin> binList = new ArrayList<ClBin>(values.length);
                    for (int i=0; i<bins.length; i++) {
                        Object value = getObject(values[i], isArithmeticAdd);
                        ClBin bin = new ClBin(bins[i], value);
                        binList.add(bin);
                    }

                    if (isArithmeticAdd) {
                        rc = cc.add(namespace, set, key, binList, cl_opts, cl_wr_opts);
                    } else if (isAppend) {
                        rc = cc.append(namespace, set, key, binList, cl_opts, cl_wr_opts);
                    } else if (isPrepend) {
                        rc = cc.prepend(namespace, set, key, binList, cl_opts, cl_wr_opts);
                    } else { // standard set
                        rc = cc.set(namespace, set, key, binList, cl_opts, cl_wr_opts);
                    }

                } else {
                    String binName = bins != null && bins.length > 0 ? bins[0] : "";
                    Object value = getObject(values[0], isArithmeticAdd);
                    if (isArithmeticAdd) {
                        rc = cc.add(namespace, set, key, binName, value, cl_opts, cl_wr_opts);
                    } else if (isAppend) {
                        rc = cc.append(namespace, set, key, binName, value, cl_opts, cl_wr_opts);
                    } else if (isPrepend) {
                        rc = cc.prepend(namespace, set, key, binName, value, cl_opts, cl_wr_opts);
                    } else {
                        rc = cc.set(namespace, set, key, binName, value, cl_opts, cl_wr_opts);
                    }
                }

                throwOnError(rc);

            } else if (method.equals("DELETE")) {
                ClWriteOptions cl_wr_opts = new ClWriteOptions();

                if (generation != 0) {
                    cl_wr_opts.set_generation(generation);
                }

                ClResultCode rc = cc.delete(namespace, set, key, cl_opts, cl_wr_opts);

                throwOnError(rc);
                
            } else {
                throw new BadMethodException("Unknown Method");
            }

        } catch (TimeoutException e) {
            responseCode = HttpServletResponse.SC_REQUEST_TIMEOUT;
            errorMsg = e.getMessage();

        } catch (BadMethodException e) {
            responseCode = HttpServletResponse.SC_METHOD_NOT_ALLOWED;
            errorMsg = e.getMessage();

        } catch (BadParamException e) {
            responseCode = HttpServletResponse.SC_BAD_REQUEST;
            errorMsg = e.getMessage();

        } catch (InternalException e) {
            responseCode = 500;
            errorMsg = e.getMessage();

        } catch (GenerationException e) {
            responseCode = HttpServletResponse.SC_CONFLICT;
            errorMsg = e.getMessage();

        } catch (NotSetException e) {
            responseCode = HttpServletResponse.SC_SERVICE_UNAVAILABLE;
            errorMsg = e.getMessage();

        } catch (ResourceNotFoundException e) {
            responseCode = HttpServletResponse.SC_NOT_FOUND;
            errorMsg = e.getMessage();
        }

        if (responseCode != HttpServletResponse.SC_OK) {
            try {
                response.sendError(responseCode, errorMsg); 

            } catch (java.io.IOException e) {
            }
        } else {
            response.setStatus(HttpServletResponse.SC_OK);
            writer.flush();
        }
    }

    private void throwOnError(ClResultCode rc) 
    	throws ResourceNotFoundException, TimeoutException, BadParamException, InternalException, GenerationException, NotSetException {
        
    	if (rc == ClResultCode.OK) {
            return;
        }

        if (rc == ClResultCode.KEY_NOT_FOUND_ERROR) {
            throw new ResourceNotFoundException("Key Not Found");
        } else if (rc == ClResultCode.TIMEOUT) {
            throw new TimeoutException("Request Timeout");
        } else if (rc == ClResultCode.PARAMETER_ERROR) {
            throw new BadParamException("Bad Parameter - Check Namespace");
        } else if (rc == ClResultCode.SERVER_ERROR) {
            throw new InternalException("Server Error");
        } else if (rc == ClResultCode.GENERATION_ERROR) {
            throw new GenerationException("Generation Conflict");
        } else if (rc == ClResultCode.NOT_SET) {
            throw new NotSetException("Key Could Not Be Set");
        }else if (rc != ClResultCode.OK) {
            throw new InternalException("");
        }
    }

    private Object getObject(String valueStr, boolean isArithmeticAdd) 
    	throws BadParamException {
    	if (! isArithmeticAdd) {
    		return valueStr;
    	}
    	
        try {
            return Integer.parseInt(valueStr);
        } catch (NumberFormatException e) {
            throw new BadParamException("Integer Value Required"); 
        }
    }
    
    private static String stripChar(String inStr, char stripChar) {
        int i = 0;
        for (i = 0; i < inStr.length(); i++) {
            if (inStr.charAt(i) != stripChar) {
                break;
            }
        }
        if (i == inStr.length())
            return inStr;
        else
            return inStr.substring(i, inStr.length());
    }

    @Override
	public void destroy() {
		if (cc != null) {
			cc.close();
			cc = null;
		}
	}
	
    /*
    private void debugInput(HttpServletRequest request, HttpServletResponse response) 
    	throws IOException {
		@SuppressWarnings("unchecked")
		Map<String, String[]> params = request.getParameterMap();
	    response.setContentType("text/html");
	    PrintWriter out = response.getWriter();
	    out.println("<html>");
	    out.println("<body>");
	
		for (Map.Entry<String,String[]> entry : params.entrySet()) {
	        out.print(entry.getKey() + ": ");
			
	        for (String value : entry.getValue()) {
		        out.print(value + ",");   	        	
	        }
		}   		
	    out.println("</body>");
	    out.println("</html>");
    }
    */
    
	private class BadParamException extends Exception
	{
		private static final long serialVersionUID = 1L;
	
		public BadParamException(String message) {
			super(message);
		}
	}
	
	private class BadMethodException extends Exception
	{
		private static final long serialVersionUID = 1L;
	
		public BadMethodException(String message) {
			super(message);
		}
	}
	
	private class ResourceNotFoundException extends Exception
	{
		private static final long serialVersionUID = 1L;
	
		public ResourceNotFoundException(String message) {
			super(message);
		}
	}
	
	private class TimeoutException extends Exception
	{
		private static final long serialVersionUID = 1L;
	
		public TimeoutException(String message) {
			super(message);
		}
	}
	
	private class InternalException extends Exception
	{
		private static final long serialVersionUID = 1L;
	
		public InternalException(String message) {
			super(message);
		}
	}
	
	private class GenerationException extends Exception
	{
		private static final long serialVersionUID = 1L;
	
		public GenerationException(String message) {
			super(message);
		}
	}
	
	private class NotSetException extends Exception
	{
		private static final long serialVersionUID = 1L;
	
		public NotSetException(String message) {
			super(message);
		}
	}
}
